#!/usr/bin/env python3


import threed_optix.api as opt
import matplotlib.pyplot as plt
import matplotlib.animation as animation


def main():
    api = opt.ThreedOptixAPI('e31aeeae-e9b0-436c-abba-735ee10cee8e')

    setups = api.get_setups()
    setup = [s for s in setups if s.name == 'setup_a'][0]
    print(f"working with '{setup}' setup")
    api.fetch(setup)

    part = setup.parts[1]
    print(f"finding the focal length of '{part}'")
    api.fetch(part)

    original_z = part.pose.position.z

    def run_simulation():
        result = api.run(setup)
        api.fetch(result)
        data = result.data
        data = data[data['hit_surface_idx'] == 3]
        return data

    fig, ax = plt.subplots(figsize=(10, 10))

    # to determine the ranges of axes
    data = run_simulation()
    minmax_x = (data['Hx'].min(), data['Hx'].max())
    minmax_y = (data['Hy'].min(), data['Hy'].max())

    areas = []
    z_positions = []
    n = 60

    def animate(i):
        ax.clear()

        print(f"running simulation for z position {part.pose.position.z} ({i+1} of {n})")
        data = run_simulation()

        ax.scatter(data['Hx'], data['Hy'])
        ax.set_title(f'z position = {part.pose.position.z}')
        ax.set_xlabel('Hx')
        ax.set_ylabel('Hy')
        ax.set_xlim(minmax_x[0], minmax_x[1])
        ax.set_ylim(minmax_y[0], minmax_y[1])

        z_positions.append(part.pose.position.z)
        area = (data['Hx'].max() - data['Hx'].min()) * (data['Hy'].max() - data['Hy'].min())
        areas.append(area)

        part.pose.position.z += 1
        api.update(part)

    anim = animation.FuncAnimation(fig, animate, frames=n)
    anim.save('spot.gif', writer='pillow')

    plt.close()

    min_area_index = areas.index(min(areas))
    print(f"z position of minimum area: {z_positions[min_area_index]}")

    fig, ax = plt.subplots(figsize=(10, 10))
    ax.plot(z_positions, areas, markevery=[min_area_index], marker='o', ms=20, mec='r', mfc='None')
    ax.set_xlabel('z position')
    ax.set_ylabel('area')
    plt.show()

    part.pose.position.z = original_z
    api.update(part)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3


import threed_optix.api as opt
import copy


def main():
    # create an instance of 'ThreedOptixAPI' passing your API key
    api = opt.ThreedOptixAPI('e31aeeae-e9b0-436c-abba-735ee10cee8e')

    # get list of all your setups (in this stage 'Setup' objects have only 'name' and 'id' properties)
    setups = api.get_setups()
    print("setups:")
    for s in setups:
        print(f"\t{s}")
    print()

    # select some setup from the list
    setup = setups[0]

    # if you want just to run a simulation, you don't need to fetch setup details;
    # just call the 'run' method of the api object
    result = api.run(setup)

    # the 'AnalysisResult' object returned by the 'run' method has no actual data,
    # only the 'url' property pointing to where the data is stored in the cloud
    # use 'fetch' method of the api object to bring the data
    api.fetch(result)
    print("simulation result:")
    print(result.data)
    print()

    # you can save the data to the disk
    result.data.to_csv('rays.csv')

    # if you want to work with parts in the setup, you must fetch the setup
    # using the 'fetch' method of the api object (you guessed correctly)
    api.fetch(setup)
    print(f"parts in '{setup}' setup")
    for p in setup.parts:
        print(f"\t{p}")
    print()

    # now you can change properties of some part,
    # but don't forget to fetch it beforehand (see the pattern?)
    part = setup.parts[0]
    api.fetch(part)

    # we want to be able to restore our part to its original state after we done with our games,
    # so we make a deep copy of the part
    original_part = copy.deepcopy(part)

    # let's change the pose of the part
    print(f"pose of local '{part}' part:              {part.pose}")
    part.pose.position.x += 33.66
    part.pose.rotation = opt.Vector3(90, 0, 45)
    print(f"pose of local '{part}' part after change: {part.pose}")

    # great, now you moved and rotated the part;
    # but wait, this changed only your local copy of the part; let's update it in the cloud
    api.update(part)

    # just to make sure that the part was updated, we will fetch it again
    api.fetch(part)
    print(f"pose of '{part}' part in the cloud:       {part.pose}")

    # and finaly, lets restore our original part in the cloud
    api.update(original_part)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages


setup(
    name='threed_optix',
    version='1.0.5',
    license='',
    author="Elika Ron",
    author_email='alikaron@3doptix.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    url='https://github.com/3doptix/3DOptix-Canvas-API',
    keywords='',
    install_requires=[
        'requests',
        'pandas',
        'matplotlib'
    ]
)
